;*******************************************************************************
;FLECHE.ASM                                                                    *
; Gestion de la fleche:                                                        *
;*******************************************************************************
;AFLECHE : Affichage de la fleche aux coordonnees courantes si elle est absente*
;   modifies : Aucun                                                           *
;                                                                              *
;EFLECHE : Effacement de la fleche si elle est presente                        *
;   modifies : Aucun                                                           *
;                                                                              *
;GFLECH1 : Renvoie dans HL les coordonnees ecran de la fleche                  *
;   modifies : HL                                                              *
;                                                                              *
;GFLECH2 : Renvoie dans HL l'adresse ecran de la fleche                        *
;   modifies : HL                                                              *
;                                                                              *
;TFLECHE : Renvoie dans A le contenu de l'octet ecran pointe par la fleche.    *
;   modifies : AF                                                              *
;                                                                              *
;MFLECHE : Deplace la fleche une fois selon l'etat du joystick.                *
;          Cette routine gere la vitesse de deplacement en la reinitialisant   *
;          si la CARRY est non nulle en entree et en l'augmentant tous les huit*
;          appels sinon.                                                       *
;          A contient l'etat du joystick                                       *
;   modifies : AF                                                              *
;                                                                              *
;*******************************************************************************
;
FLBUF    DEFS 18*4
FLROW    DEFB 199
FLCOL    DEFB 0
FLADD    DEFS 2
FLSTATE  DEFS 1
;
;
;AFLECHE
;
AFLECHE
         PUSH AF
         PUSH BC
         PUSH DE
         PUSH HL
         PUSH IX
         PUSH IY
         LD   A,(FLSTATE)
         OR   A
         JR   NZ,AFLECHE0
         LD   H,A
         LD   D,A
         LD   A,(FLCOL)
         LD   L,A
         ADD  HL,HL
         EX   DE,HL
         LD   A,(FLROW)
         LD   L,A
         CALL #BC1D          ;SCR DOT POSITION
         LD   (FLADD),HL
         CALL SCRMEM
         CALL #BD19          ;MC WAIT FLYBACK
         LD   HL,(FLADD)
         CALL FLON
AFLECHE0 POP  IY
         POP  IX
         POP  HL
         POP  DE
         POP  BC
         POP  AF
         RET
;
;
;EFLECHE
;
EFLECHE
         PUSH AF
         PUSH BC
         PUSH DE
         PUSH HL
         LD   A,(FLSTATE)
         OR   A
         JR   Z,EFLECHE0
         CALL #BD19          ;MC WAIT FLYBACK
         LD   HL,(FLADD)
         CALL MEMSCR
EFLECHE0 POP  HL
         POP  DE
         POP  BC
         POP  AF
         RET
;
;
;GFLECH1
;
GFLECH1
         LD   HL,(FLROW)
         RET
;
;
;GFLECH2
;
GFLECH2
         LD   HL,(FLADD)
         RET
;
;
;TFLECHE
;
TFLECHE
         PUSH HL
         LD   HL,(FLADD)
         LD   A,(HL)
         POP  HL
         RET
;
;
;MFLECHE
;
FLVITFL  DEFS 1
FLVIT    DEFS 1
;
MFLECHE
         PUSH BC
         PUSH DE
         PUSH HL
         PUSH IX
         PUSH IY
         JR   NC,NOTOFF
         LD   A,1
         LD   (FLVIT),A
         LD   (FLVITFL),A
NOTOFF   CALL GETJOY
         PUSH AF
         LD   D,0
         LD   C,A
         LD   A,(FLVIT)
         LD   B,A
         LD   HL,(FLADD)
         RR   C
         JR   NC,NOTUP
         INC  D
         LD   A,(FLROW)
         ADD  A,B
         CP   200
         JR   C,UPNOCAR
         LD   A,199
UPNOCAR  LD   (FLROW),A
NOTUP    RR   C
         JR   NC,NOTDN
         INC  D
         LD   A,(FLROW)
         SUB  B
         JR   C,DNCAR
         CP   4
         JR   NC,DNNOCAR
DNCAR    LD   A,4
DNNOCAR  LD   (FLROW),A
NOTDN    RR   C
         JR   NC,NOTLT
         INC  D
         LD   A,(FLCOL)
         SUB  B
         JR   NC,LTNOCAR
         LD   A,0
LTNOCAR  LD   (FLCOL),A
NOTLT    RR   C
         JR   NC,NOTRT
         INC  D
         LD   A,(FLCOL)
         ADD  A,B
         CP   154
         JR   C,RTNOCAR
         LD   A,153
RTNOCAR  LD   (FLCOL),A
NOTRT    LD   A,D
         OR   A
         JR   NZ,CHNGVIT
         LD   A,1
         LD   (FLVIT),A
         LD   (FLVITFL),A
         JR   NOMOVE
CHNGVIT  LD   IX,FLVITFL
         RLC  (IX+00)
         JR   NC,TOMOVE
         INC  (IX+01)
         BIT  3,(IX+01)
         JR   Z,TOMOVE
         RRC  (IX+01)
TOMOVE   XOR  A
         LD   H,A
         LD   D,A
         LD   A,(FLCOL)
         LD   L,A
         ADD  HL,HL
         EX   DE,HL
         LD   A,(FLROW)
         LD   L,A
         CALL #BC1D          ;SCR DOT POSITION
         PUSH HL
         CALL #BD19          ;MC WAIT FLYBACK
         DI
         LD   HL,(FLADD)
         CALL MEMSCR
         POP  HL
         LD   (FLADD),HL
         CALL SCRMEM
         LD   HL,(FLADD)
         CALL FLON
         EI
NOMOVE   POP  AF
         POP  IY
         POP  IX
         POP  HL
         POP  DE
         POP  BC
         RET
;
;
;FLON
;
;
;Dessin de la fleche (72 Octets)
;
SWORD0
         DEFB #0F,#00,#00,#00
         DEFB #4F,#0A,#00,#00
         DEFB #4F,#0A,#00,#00
         DEFB #4F,#0A,#00,#00
         DEFB #4F,#AF,#00,#00
         DEFB #5F,#8F,#00,#00
         DEFB #05,#8F,#00,#00
         DEFB #05,#8F,#00,#00
         DEFB #05,#8F,#0F,#00
         DEFB #05,#FF,#3F,#0A
         DEFB #05,#BF,#2F,#00
         DEFB #05,#3F,#0A,#00
         DEFB #1F,#7F,#0A,#00
         DEFB #1F,#0F,#AF,#00
         DEFB #05,#05,#AF,#00
         DEFB #00,#05,#AF,#00
         DEFB #00,#05,#AF,#00
         DEFB #00,#00,#0A,#00
;
SWORD1
         DEFB #05,#0A,#00,#00
         DEFB #05,#8F,#00,#00
         DEFB #05,#8F,#00,#00
         DEFB #05,#8F,#00,#00
         DEFB #05,#DF,#0A,#00
         DEFB #05,#EF,#0A,#00
         DEFB #00,#4F,#0A,#00
         DEFB #00,#4F,#0A,#00
         DEFB #00,#4F,#0F,#0A
         DEFB #00,#5F,#BF,#2F
         DEFB #00,#5F,#3F,#0A
         DEFB #00,#1F,#2F,#00
         DEFB #05,#3F,#AF,#00
         DEFB #05,#2F,#5F,#0A
         DEFB #00,#0A,#5F,#0A
         DEFB #00,#00,#5F,#0A
         DEFB #00,#00,#5F,#0A
         DEFB #00,#00,#05,#00
;
;
FLON
         LD   A,1
         LD   (FLSTATE),A
         LD   DE,SWORD0
         LD   A,(FLCOL)
         RRA
         JR   NC,FLONEVEN
         LD   DE,SWORD1
FLONEVEN LD   A,(FLROW)
         INC  A
         CP   19
         JR   C,FLON10
         LD   A,18
FLON10   LD   C,A
FLON0    LD   B,4
         PUSH HL
FLON1    LD   A,(DE)
         RRCA
         RRCA
         RRCA
         RRCA
         AND  #F0
         CPL
         AND  (HL)
         LD   (HL),A
         LD   A,(DE)
         OR   (HL)
         LD   (HL),A
         INC  DE
         INC  HL
         DJNZ FLON1
         POP  HL
         CALL NEXTLINE
         DEC  C
         JR   NZ,FLON0
         RET
;
;
;SCRMEM
;
SCRMEM
         LD   DE,FLBUF
         LD   B,18
SCRMEM0  PUSH BC
         PUSH HL
         LD   BC,4
         LDIR
         POP  HL
         CALL NEXTLINE
         POP  BC
         DJNZ SCRMEM0
         RET
;
;
;MEMSCR
;
MEMSCR   XOR  A
         LD   (FLSTATE),A
         LD   DE,FLBUF
         LD   B,18
MEMSCR0  PUSH BC
         PUSH HL
         LD   BC,4
         EX   DE,HL
         LDIR
         EX   DE,HL
         POP  HL
         CALL NEXTLINE
         POP  BC
         DJNZ MEMSCR0
         RET
;
