;
;*******************************************************************************
;MUSINT : Interpreteur de musique sous interruptions                           *
;   (c)1988 B.RIVE                                                             *
;*******************************************************************************
;BRUIT  :     Execute un des bruitages (Numero dans A)                         *
;*******************************************************************************
;LOADMUS:     Charge et lance la musique indiquee par HL                       *
;*******************************************************************************
;ONOFF: Suspend la musique en cours ou relance la musique suspendue            *
;*******************************************************************************
;OFF  : Stoppe la musique en cours                                             *
;*******************************************************************************
;ON   : Lance la musique pointee par HL                                        *
;*******************************************************************************
;
BRUIT
         PUSH BC
         PUSH DE
         PUSH HL
         PUSH IX
         PUSH IY
         PUSH AF
         CALL MUSBUF+3
         POP  AF
         PUSH AF
         INC  A
         LD   IX,DEFINE
         LD   (IX+02),A
         LD   A,2
         CALL MUSBUF
         POP  AF
         POP  IY
         POP  IX
         POP  HL
         POP  DE
         POP  BC
         RET

DEFINE   DEFW 1
         DEFW 0
;
LOADMUS
         PUSH DE
         LD   DE,MUSBUF
         CALL READFILE
         LD   HL,MUSBUF
         CALL ON
         POP  DE
         RET
;
ONOFF
         PUSH AF
         LD   A,(BOUTON)
         OR   A
         JR   NZ,OFF1
         PUSH HL
         LD   HL,(PC)
         JR   ON1
OFF
         PUSH AF
OFF1     PUSH HL
         PUSH BC
         PUSH DE
         LD   HL,EBLOCK
         CALL #BCF8          ;KL DEL SYNCHRONOUS
         LD   HL,TICKER
         CALL #BCEC          ;KL DEL TICKER
         XOR  A
         LD   (BOUTON),A
         CALL #BCA7          ;SOUND RESET
MUSDEP   POP  DE
         POP  BC
         POP  HL
         POP  AF
         RET
ON
         PUSH AF
         CALL OFF
         PUSH HL
ON1      PUSH BC
         PUSH DE
         LD   (PC),HL
         CALL #BCA7          ;SOUND RESET
         LD   HL,EBLOCK
         LD   B,#81
         LD   DE,MUSINT
         CALL #BCEF          ;KL INIT EVENT
         LD   HL,TICKER
         LD   DE,10
         LD   BC,10
         CALL #BCE9          ;KL ADD TICKER
         LD   A,1
         LD   (BOUTON),A
         JR   MUSDEP
;
BOUTON   DEFS 1
TICKER   DEFS 6
EBLOCK   DEFS 7
;
;
;Sequenceur musical
;
;Interpretation du bloc de donnees:
;-1er octet:
;   -Bit 7 = 0 : Note
;        Bit 0 : canal A
;        Bit 1 : canal B
;        Bit 2 : canal C
;        Bit 3 : rendez-vous avec canal A
;        Bit 4 : rendez-vous avec canal B
;        Bit 5 : rendez-vous avec canal C
;        Bit 6 : hold jusqu'a release
;      2 eme octet: Numero de la note
;      3 eme octet: Duree de la note
;      4 eme octet:
;        Bits 0->4 : Periode de bruit
;        Bits 5->7 : Numero de l'instrument
;   -Bit 7 = 1 : Commande
;        Bits 4->6 : Numero de la commande
;             -0 -> Definit une enveloppe de volume:
;                  Bits 0->2 : Nombre de sections de l'amplitude
;                  2eme octet : Numero de l'enveloppe
;                  octets suivants : 3 octets par section:
;                    -si bit 7 du premier a 0 : envoloppe soft:
;                       nombre de pas
;                       taille d'un pas
;                       duree d'un pas
;                    -si bit 7 du premier a 1 : enveloppe hard:
;                       numero d'enveloppe
;                       periode de l'enveloppe sur deux octets
;             -1 -> Definit une enveloppe de tonalite:
;                  cf. ci-dessus
;             -2 -> Definit un instrument:
;                  Bits 0->2 : Numero de l'instrument
;                  2eme octet: Volume initial
;                  3eme octet: bits 0->3 = numero enveloppe de volume
;                              bits 4->7 = numero enveloppe de tonalite
;             -3 -> Instructions speciales:
;                  -Bit 3 = 1 -> Transposition:
;                       bits 0->2 : hauteur signee de la transposition
;                  -Bit 3 = 0 -> Commande:
;                       bits 0->2 : numero de la commande
;                            -0 -> Goto:
;                                 octets 2 et 3: offset signe du saut
;                            -1 -> Gosub:
;                                 octets 2 et 3: offset signe de l'appel
;                                 adresse de retour placce dans RETADD
;                            -2 -> Return:
;                                 l'adresse de retour est dans RETADD
;                            -3 -> Tempo:
;                                 octet 2: valeur du nouveau tempo
;                            -4,5,6 -> End:
;                                 fin du morceau;bloque le sequenceur
;                            -7 -> Normal:
;                                 Annule la transposition
;             -4 -> Chargement d'un des compteurs:
;                  Bit 0 : Indique lequel des deux compteurs est concerne
;                  2eme octet: valeur a mettre dans le compteur
;             -5 -> Test d'un des compteurs:
;                  Bit 0 : Indique lequel des deux compteurs est concerne
;                  2eme octet: valeur a comparer au compteur
;                  octets 3 et 4: offset du saut a effectuer si egalite
;             -6 -> Decrement d'un des compteurs:
;                  Bit 0 : Indique lequel des deux compteurs est concerne
;             -7 -> Increment d'un des compteurs:
;                  Bit 0 : Indique lequel des deux compteurs est concerne
;
;VARIABLES
;
PC      DEFS    2       ;Pointeur du sequenceur
RETADD  DEFS    2       ;Adresse de retour pour un gosub
WSPC    DEFS    16      ;Buffer de donnees d'une note ou d'une enveloppe
CPT     DEFS    2       ;Deux compteurs
TRANS   DEFS    1       ;Transposition courante
TEMPO   DEFS    1       ;Tempo courant
TINST   DEFS    16      ;Table des instruments
;
;CONSTANTES
;Table des periodes des notes
;
        FREE
        DBXON
NTAB    !3822,!3608,!3405,!3214,!3034
        !2863,!2703,!2551,!2408,!2273
        !2145,!2025,!1911,!1804,!1703
        !1607,!1517,!1432,!1351,!1276
        !1204,!1136,!1073,!1012,!0956
        !0956,!0902,!0851,!0804,!0758
        !0716,!0676,!0638,!0602,!0568
        !0536,!0506,!0478,!0451,!0426
        !0402,!0379,!0358,!0338,!0319
        !0301,!0284,!0268,!0253,!0239
        !0225,!0213,!0201,!0190,!0179
        !0169,!0159,!0150,!0142,!0134
        !0127,!0119,!0113,!0106,!0100
        !0095,!0089,!0084,!0080,!0075
        !0071,!0067,!0063,!0060,!0056
        !0053,!0050,!0047,!0045,!0042
        !0040,!0038,!0036,!0034,!0032
        !0030,!0028,!0027,!0025,!0024
        !0022,!0021,!0020,!0019,!0018
        !0017,!0016
        DBXOFF
;
MUSINT  LD      HL,(PC)
        LD      A,H
        OR      L
        RET     Z
        LD      A,(HL)
        BIT     7,A
        JP      NZ,COM
        AND     7
        PUSH    HL
        CALL    #BCAD
        POP     HL
        AND     7
        RET     Z

        PUSH    IX
        LD      IX,WSPC
        LD      A,(HL)
        LD      (IX+0),A
        INC     HL
        LD      A,(HL)
        AND     A
        JR      Z,A00
        LD      B,A
        LD      A,(TRANS)
        ADD     A,B
        PUSH    HL
        LD      L,A
        LD      H,0
        ADD     HL,HL
        LD      DE,NTAB-2
        ADD     HL,DE
        LD      A,(HL)
        LD      (IX+3),A
        INC     HL
        LD      A,(HL)
        LD      (IX+4),A
        POP     HL
        JR      A03
A00     LD      (IX+3),0
        LD      (IX+4),0
A03     INC     HL
        LD      C,(HL)
        LD      B,0
        PUSH    HL
        LD      H,B
        LD      L,B
        LD      A,(TEMPO)
A01     SRL     A
        JR      NC,A02
        ADD     HL,BC
A02     SLA     C
        OR      A
        JR      NZ,A01
        LD      (IX+7),L
        LD      (IX+8),H
        POP     HL
        INC     HL
        LD      A,(HL)
        AND     31
        LD      (IX+5),A
        LD      A,(HL)
        INC     HL
        LD      (PC),HL
        RLCA
        RLCA
        RLCA
        AND     7
        ADD     A,A
        LD      L,A
        LD      H,0
        LD      DE,TINST
        ADD     HL,DE
        LD      A,(HL)
        LD      (IX+6),A
        INC     HL
        XOR     A
        RLD
        LD      (IX+1),A
        RLD
        LD      (IX+2),A
        RLD
        LD      HL,WSPC
        CALL    #BCAA
        POP     IX
        JP      MUSINT

COM
        RRCA
        RRCA
        RRCA
        AND     #0E
        LD      DE,CTAB

JPTAB   PUSH    HL
        LD      L,A
        LD      H,0
        ADD     HL,DE
        LD      E,(HL)
        INC     HL
        LD      H,(HL)
        LD      L,E
        EX      (SP),HL
        RET

CTAB    DEFW    ENV
        DEFW    ENT
        DEFW    INST
        DEFW    SPEC
        DEFW    LOAD
        DEFW    TEST
        DEFW    DECR
        DEFW    INCR

ENV     LD      DE,#BCBC
        JR      A04

ENT     LD      DE,#BCBF
A04     LD      (A05+1),DE
        LD      A,(HL)
        AND     7
        INC     HL
        BIT     7,(HL)
        JR      Z,A09
        SET     7,A
A09     LD      (WSPC),A
        AND     #7F
        LD      B,A
        LD      A,(HL)
        AND     #7F
        PUSH    AF
        INC     HL
        LD      A,B
        ADD     A,A
        ADD     A,B
        JR      Z,A06
        LD      C,A
        LD      B,0
        LD      DE,WSPC+1
        LDIR
A06     LD      (PC),HL
        POP     AF
        LD      HL,WSPC
A05     CALL    0
        JP      MUSINT

INST    LD      A,(HL)
        AND     7
        ADD     A,A
        PUSH    HL
        LD      L,A
        LD      H,0
        LD      DE,TINST
        ADD     HL,DE
        EX      DE,HL
        POP     HL
        INC     HL
        LD      BC,2
        LDIR
        LD      (PC),HL
        JP      MUSINT

A07     LD      A,(HL)
        INC     HL
        RRCA
        LD      DE,CPT
        RET     NC
        INC     DE
        RET

LOAD    CALL    A07
        LD      A,(HL)
        LD      (DE),A
        INC     HL
        LD      (PC),HL
        JP      MUSINT

INCR    CALL    A07
        LD      (PC),HL
        EX      DE,HL
        INC     (HL)
        JP      MUSINT

DECR    CALL    A07
        LD      (PC),HL
        EX      DE,HL
        DEC     (HL)
        JP      MUSINT

TEST    CALL    A07
        LD      A,(DE)
        CP      (HL)
        INC     HL
        JR      Z,JUMP
        INC     HL
        INC     HL
        LD      (PC),HL
        JP      MUSINT

JUMP    LD      E,(HL)
        INC     HL
        LD      D,(HL)
        INC     HL
        ADD     HL,DE
        LD      (PC),HL
        JP      MUSINT

SPEC    LD      A,(HL)
        BIT     3,A
        JR      NZ,TRAN
        AND     7
        ADD     A,A
        LD      DE,SPTAB
        JP      JPTAB

SPTAB   DEFW    GOTO
        DEFW    GOSUB
        DEFW    RETURN
        DEFW    TEMP
        DEFW    MUSFIN
        DEFW    MUSFIN
        DEFW    MUSFIN
        DEFW    NORM

TRAN    AND     7
        BIT     2,A
        JR      Z,A08
        OR      #F8
A08     INC     HL
        LD      (PC),HL
        LD      HL,TRANS
        ADD     A,(HL)
        LD      (HL),A
        JP      MUSINT

GOTO    INC     HL
        JR      JUMP

GOSUB   INC     HL
        INC     HL
        INC     HL
        LD      (RETADD),HL
        DEC     HL
        DEC     HL
        JR      JUMP

RETURN  LD      HL,(RETADD)
        LD      (PC),HL
        JP      MUSINT

TEMP    INC     HL
        LD      A,(HL)
        INC     HL
        LD      (PC),HL
        LD      (TEMPO),A
        JP      MUSINT

MUSFIN  LD      HL,0
        LD      (PC),HL
        RET

NORM    INC     HL
        LD      (PC),HL
        XOR     A
        LD      (TRAN),A
        JP      MUSINT
