;*******************************************************************************
;   SCR.ASM                                                                    *
;        Routines de gestion de l'ecran                                        *
;*******************************************************************************
;LOADSCR  : Charge une image a partir du disque et l'affiche a l'ecran
;   ENTRY : HL-> Adresse du nom du fichier sur 8 octets
;   Modifies : Aucun
;SAVESCR  : Sauve l'ecran 16000 octets en #0040
;   Modifies : Aucun
;RESTSCR  : Restore l'ecran
;   Modifies : Aucun
;SETPAL   : Met en place une palette
;   ENTRY : HL-> Adresse de la palette
;   Modifies : HL
;BLACKOUT : Met toutes les encres a 0.
;   Modifies :Aucun
;DECOMP   : Decompactage d'une image
;   INPUT : HL-> Adresse de l'image compactee
;AFFIMA   : Affichage d'une image
;   INPUT : HL-> Adresse de l'image compactee
;SETFLASH : Mise en place du clignotement des couleurs 4 et 5
;*******************************************************************************
;
;SETFLASH
;
SETFLASH
         PUSH AF
         PUSH BC
         PUSH HL
         LD   HL,#0A0A
         CALL #BC3E          ;SCR SET FLASHING
         LD   A,4
         LD   BC,#180F
         CALL #BC32          ;SCR SET INK
         LD   A,5
         LD   BC,#0F18
         CALL #BC32
         POP  HL
         POP  BC
         POP  AF
         RET
;
;LOADSCR
;
LOADSCR
         PUSH BC
         PUSH DE
         LD   DE,BUFIMA
         CALL READFILE
         CALL BLACKOUT
         LD   HL,BUFIMA
         CALL DECOMP
         LD   HL,BUFIMA
         CALL SETPAL
         POP  DE
         POP  BC
         RET
;
;
;AFFIMA
;
AFFIMA
         CALL BLACKOUT
         CALL DECOMP
         CALL SETPAL
         RET
;
;
;DECOMP
;
DECOMP
         PUSH HL
         PUSH DE
         PUSH BC
         PUSH AF
         LD   DE,16
         ADD  HL,DE
         LD   DE,#C000
DECOMP0  BIT  6,D
         JR   Z,DECOMP1
         LD   B,1
         LD   A,(HL)
         INC  HL
         CP   01
         JR   NZ,DECOMP2
         LD   B,(HL)
         INC  HL
         LD   A,(HL)
         INC  HL
DECOMP2  LD   (DE),A
         INC  DE
         DJNZ DECOMP2
         JR   DECOMP0
DECOMP1  POP  AF
         POP  BC
         POP  DE
         POP  HL
         RET
;
;
;SETPAL
;
PALFIX
         DEFB #14,#03,#10,#0F,#0D
;
SETPAL
         PUSH DE
         PUSH BC
         PUSH AF
         PUSH HL
         LD   A,0
         LD   B,11
         CALL CHINK
         LD   HL,PALFIX
         LD   B,5
         CALL CHINK
         POP  HL
         POP  AF
         POP  BC
         POP  DE
         RET
CHINK
         PUSH BC
         PUSH AF
         PUSH HL
         LD   C,(HL)
         LD   B,C
         CALL #BC32
         POP  HL
         POP  AF
         POP  BC
         INC  HL
         INC  A
         DJNZ CHINK
         RET
;
;
;BLACKOUT
;
BLACKOUT
         PUSH AF
         PUSH BC
         PUSH DE
         PUSH HL
         LD   BC,0
         CALL #BC38
         LD   C,2
BLACKOU0 LD   A,4
         CALL MILPAUSE
         CALL COLDOWN
         DEC  C
         JR   NZ,BLACKOU0
         POP  HL
         POP  DE
         POP  BC
         POP  AF
         RET
;
;
;COLDOWN
;
COLDOWN
         PUSH HL
         PUSH DE
         PUSH BC
         PUSH AF
         LD   A,#10
COLD0    PUSH AF
         CALL #BC35          ;SCR GET INK
         LD   A,B
         LD   D,9
         CALL DIVI
         OR   A
         JR   Z,COLD1
         DEC  A
COLD1    CALL MUL
         LD   C,A
         LD   A,B
         CALL MOD
         LD   B,A
         LD   D,3
         CALL DIVI
         OR   A
         JR   Z,COLD2
         DEC  A
COLD2    CALL MUL
         ADD  A,C
         LD   C,A
         LD   A,B
         CALL MOD
         OR   A
         JR   Z,COLD3
         DEC  A
COLD3    ADD  A,C
         LD   C,A
         LD   B,A
         POP  AF
         PUSH AF
         CALL #BC32
         POP  AF
         DEC  A
         JP   P,COLD0
         POP  AF
         POP  BC
         POP  DE
         POP  HL
         RET
;
;MOD: A= A MOD D
;
MOD
         SUB  D
         JR   NC,MOD
         ADD  A,D
         RET
;
;DIVI: A= A DIV D
;
DIVI
         PUSH DE
         LD   E,0
DIVI0    SUB  D
         INC  E
         JR   NC,DIVI0
         DEC  E
         LD   A,E
         POP  DE
         RET
;
;MUL: A= A MUL D
;
MUL
         OR   A
         RET  Z
         PUSH BC
         LD   B,A
         XOR  A
MUL0     ADD  A,D
         DJNZ MUL0
         POP  BC
         RET
;
;
;SAVESCR
;
SAVESCR
         PUSH HL
         PUSH DE
         PUSH BC
         PUSH AF
         LD   A,200
         LD   HL,#C000
         LD   DE,BUFIMA+16
SAVESCR0 PUSH HL
         LD   BC,80
         LDIR
         POP  HL
         CALL NEXTLINE
         DEC  A
         JR   NZ,SAVESCR0
         POP  AF
         POP  BC
         POP  DE
         POP  HL
         RET
;
;
;RESTSCR
;
RESTSCR
         PUSH AF
         PUSH BC
         PUSH DE
         PUSH HL
         LD   A,200
         LD   HL,BUFIMA+16
         LD   DE,#C000
RESTSCR0 PUSH DE
         LD   BC,80
         LDIR
         POP  DE
         EX   DE,HL
         CALL NEXTLINE
         EX   DE,HL
         DEC  A
         JR   NZ,RESTSCR0
         POP  HL
         POP  DE
         POP  BC
         POP  AF
         RET
