;*******************************************************************************
;TXT.ASM
;   Ensemble des routines d'affichage de texte.
;*******************************************************************************
;CENTRE   :   Centrage d'un texte (le curseur doit etre deja positionne)
;   INPUT :   HL-> Adresse du texte
;             A -> Largeur du champ
;*******************************************************************************
;AFFMESS  :   Affichage d'un texte place apres l'appel
;   Modifies: Aucun
;*******************************************************************************
;AFFMEBIS :   Affichage d'un texte
;   INPUT :   HL-> Adresse du texte
;   OUTPUT:   HL-> Adresse de l'octet apres le #FF de fin
;   Modifies: HL
;*******************************************************************************
;MENUTEXT :   Affichage d'un menu et selection d'une option
;   INPUT :   HL-> Adresse du texte
;             Variables dans la pile
;   OUTPUT:   A -> Indice de l'option choisie (0->Nbchp)
;             HL-> Adresse de l'octet suivant le texte
;   Modifies: AF,HL
;*******************************************************************************
;MENUSEL  :   Selection d'une option selon le menu defini par MENUTEXT
;   OUTPUT:   A -> Indice de l'option choisie
;   Modifies: AF
;*******************************************************************************
;
;CENTRE
;
CENTRE
         PUSH HL
         PUSH BC
         PUSH AF
         LD   B,0
CENTRE0  LD   A,(HL)
         INC  A
         JR   Z,CENTRE1
         INC  B
         INC  HL
         JR   CENTRE0
CENTRE1  POP  AF
         SUB  B
         SRL  A
         JR   Z,CENTRE3
         LD   B,A
         LD   A," "
CENTRE2  CALL #BB5A
         DJNZ CENTRE2
CENTRE3  POP  BC
         POP  HL
         RET
;
;AFFMESS
;Le texte est apres l'appel et doit se terminer par #FF
;
AFFMESS
         EX   (SP),HL
         PUSH AF
AFFMESS1 LD   A,(HL)
         INC  HL
         CP   #FF
         JR   Z,AFFMESS2
         CALL #BB5A          ;TXT OUTPUT
         JR   AFFMESS1
AFFMESS2 POP  AF
         EX   (SP),HL
         RET
;
;
;AFFMEBIS
;   Affiche le message pointe par hl et termine par #FF
;
AFFMEBIS
         PUSH AF
AFFMEBI0 LD   A,(HL)
         INC  HL
         CP   #FF
         JR   Z,AFFMEBI1
         CALL #BB5A
         JR   AFFMEBI0
AFFMEBI1 POP  AF
         RET
;
;
;MENUTEXT
;Affichage d'un texte et selection d'une option si au moins une dans le texte
;Le texte est pointe par HL
;Les variables sont dans la pile
;
CHAMPS   DEFS 32             ;4 octets pour definir chaque champ
NBCHP    DEFS 1              ;Compteur de delimiteur de champs (2*Nbe de champs)
STACK    DEFS 2
LINEBUFF DEFS 80
NBCHAR   DEFS 25
TXTCOL   DEFS 1
TXTROW   DEFS 1
TXTNBCOL DEFS 1
TXTNBROW DEFS 1
TOSAVE   DEFB 0
RETSIEGE DEFB 0
;
MENUTEXT
         LD   A,1
         LD   (TOSAVE),A
MENUTEX0
         LD   (STACK),SP
         PUSH DE
         PUSH IX
         LD   DE,(NBCHAR)
         XOR  A
         LD   (NBCHP),A
         LD   A,(HL)
         LD   (TXTCOL),A
         INC  HL
         LD   A,(HL)
         LD   (TXTROW),A
         INC  HL
         LD   A,(HL)
         LD   (TXTNBCOL),A
         INC  HL
         PUSH HL
         LD   IX,(STACK)
         INC  IX
         INC  IX
         LD   (STACK),IX
         CALL TXTCALC
         LD   (TXTNBROW),A
         LD   A,(TOSAVE)
         OR   A
         CALL NZ,MENUSAVE
         CALL TXTCADRE
         POP  HL
         LD   IX,(STACK)
         CALL TXTAFF
         LD   (STACK),IX
         POP  IX
         POP  DE
         POP  HL
         LD   SP,(STACK)
         PUSH HL
         XOR  A
         LD   (TOSAVE),A
         LD   A,(NBCHP)
         OR   A
         RET  Z
         LD   A,(RETSIEGE)
         OR   A
         RET  NZ
MENUTEX1 CALL AFLECHE
         CALL MENUSEL
         PUSH AF
         CALL EFLECHE
         POP  AF
         RET
;
;TXTCALC
;
TXTCALC
         PUSH BC
         PUSH DE
         PUSH IY
         CALL FFBUFF
         LD   DE,NBCHAR
         LD   C,0
         LD   A,(TXTNBCOL)
         LD   B,A
TXTCALC0 PUSH BC
         CALL ACTBUFF
         INC  B
         LD   C,0
TXTCALC1 CALL NXTCHAR
         CP   #FF
         JR   Z,TXTCALC4
         CP   #F9
         JR   Z,TXTCALC3
         INC  C
         DJNZ TXTCALC1
TXTCALC2 CALL PRVCHAR
         DEC  C
         CP   " "
         JR   NZ,TXTCALC2
         CALL NXTCHAR
TXTCALC3 LD   A,C
         LD   (DE),A
         INC  DE
         POP  BC
         INC  C
         JR   TXTCALC0
TXTCALC4 LD   A,C
         LD   (DE),A
         POP  BC
         LD   A,C
         INC  A
         POP  IY
         POP  DE
         POP  BC
         RET
;
;FFBUFF
;
FFBUFF
         LD   DE,LINEBUFF
         PUSH DE
         LD   A,#FF
         LD   B,80
FFBUFF0  LD   (DE),A
         INC  DE
         DJNZ FFBUFF0
         POP  IY
         RET
;
;NXTCHAR
;
NXTCHAR
NXTCHAR0 LD   A,(IY+00)
         CP   #FF
         JR   Z,NXTCHAR1
         INC  IY
         CP   #FE
         JR   Z,NXTCHAR0
         RET
NXTCHAR1 LD   A,(HL)
         INC  HL
         CP   #F0
         JR   NC,NXTCHAR3
         LD   (IY+00),A
         INC  IY
         RET
NXTCHAR3 CP   #FF
         RET  Z
         CP   #F9
         RET  Z
         CP   #FE
         JR   Z,NXTCHAR1
         PUSH HL
         CALL GETVAR
         CP   #FB
         CALL Z,CONVD16B
         CP   #FC
         CALL Z,CONVD16
         PUSH IY
NXTCHAR4 LD   A,(HL)
         CP   #FF
         JR   Z,NXTCHAR5
         LD   (IY+00),A
         INC  IY
         INC  HL
         JR   NXTCHAR4
NXTCHAR5 POP  IY
         POP  HL
         JR   NXTCHAR0
;
;PRVCHAR
;
PRVCHAR
         DEC  IY
         LD   A,(IY+00)
         RET
;
;
;TXTAFF
;
TXTAFF
         PUSH DE
         PUSH BC
         PUSH IY
         PUSH HL
         LD   A,(TXTCOL)
         LD   H,A
         INC  H
         LD   A,(TXTROW)
         LD   L,A
         INC  L
         CALL #BB75          ;SCR CHAR POSITION
         POP  HL
         CALL FFBUFF
         LD   DE,NBCHAR
TXTAFF0  CALL ACTBUFF
         LD   A,(DE)
         OR   A
         JR   Z,TXTAFF3
         LD   B,A
         LD   A,(TXTNBCOL)
         SUB  B
         SRL  A
TXTAFF1  OR   A
         JR   Z,TXTAFF2
         CALL RGHT
         DEC  A
         JR   TXTAFF1
TXTAFF2  CALL EXECHAR
         DJNZ TXTAFF2
TXTAFF3  LD   A,(IY+00)
         CP   #FF
         JR   Z,TXTAFF4
         CP   #20
         JR   NZ,TXTAFF5
         INC  IY
         JR   TXTAFF5
TXTAFF4  LD   A,(HL)
         CP   #20
         JR   NZ,TXTAFF5
         INC  HL
TXTAFF5  CALL NZ,EXECHAR
         CP   #FF
         JR   Z,TXTAFF6
         INC  DE
         CALL CR
         JR   TXTAFF0
TXTAFF6  POP  IY
         POP  BC
         POP  DE
         RET
;
;RGHT
;
RGHT
         PUSH AF
         LD   A,9
         CALL #BB5A
         POP  AF
         RET
;
;CR
;
CR
         PUSH HL
         PUSH AF
         LD   A,10
         CALL #BB5A
         LD   A,(TXTCOL)
         INC  A
         CALL #BB6F          ;TXT SET COLUMN
         POP  AF
         POP  HL
         RET
;
;ACTBUFF
;
ACTBUFF
         PUSH AF
         PUSH BC
         PUSH HL
         LD   HL,LINEBUFF
         LD   B,80
ACTBUFF0 LD   A,(IY+00)
         CP   #FF
         JR   Z,ACTBUFF1
         LD   (HL),A
         INC  HL
         DEC  B
         INC  IY
         JR   ACTBUFF0
ACTBUFF1 LD   (HL),#FF
         INC  HL
         DJNZ ACTBUFF1
         LD   IY,LINEBUFF
         POP  HL
         POP  BC
         POP  AF
         RET
;
;EXECHAR
;
EXECHAR
EXECHAR0 LD   A,(IY+00)
         INC  IY
         CP   #FF
         JR   NZ,EXECHAR1
         DEC  IY
         LD   A,(HL)
         INC  HL
EXECHAR1 CP   #F0
         JR   NC,EXECHAR3
         CALL #BB5A
         RET
EXECHAR3 CP   #FE
         JR   NZ,EXECHAR6
         CALL DELIM
         JR   EXECHAR0
EXECHAR6 CP   #FF
         RET  Z
         CP   #F9
         RET  Z
         PUSH HL
         CALL GETVAR
         CP   #FB
         CALL Z,CONVD16B
         CP   #FC
         CALL Z,CONVD16
         PUSH IY
EXECHAR4 LD   A,(HL)
         CP   #FF
         JR   Z,EXECHAR5
         LD   (IY+00),A
         INC  IY
         INC  HL
         JR   EXECHAR4
EXECHAR5 POP  IY
         POP  HL
         JR   EXECHAR0
;
;DELIM
;
DELIM
         PUSH HL
         PUSH DE
         LD   HL,NBCHP
         LD   A,(HL)
         PUSH AF
         INC  (HL)
         CALL #BB78          ;TXT GET CURSOR
         POP  AF
         RRA
         JR   NC,DELIM0
         DEC  H
DELIM0   RLA
         LD   DE,CHAMPS
         CALL WRTAB16
         POP  DE
         POP  HL
         RET
;
;AFFD16
;Affiche le nombre decimal contenu par HL
;
AFFD16
         PUSH HL
         PUSH AF
         PUSH DE
         LD   A," "
         LD   (DECHAR+1),A
         LD   DE,10000
         CALL AFFD160
         LD   DE,1000
         CALL AFFD160
         LD   DE,100
         CALL AFFD160
         LD   DE,10
         CALL AFFD160
         LD   DE,1
         LD   A,"0"
         LD   (DECHAR+1),A
         CALL AFFD160
         POP  DE
         POP  AF
         POP  HL
         RET
AFFD160  LD   A,#FF
         OR   A
AFFD161  SBC  HL,DE
         INC  A
         JR   NC,AFFD161
         ADD  HL,DE
DECHAR   ADD  A," "
         CP   " "
         JP   Z,#BB5A             ;TXT OUTPUT
         CP   " "+10
         JP   NC,#BB5A            ;TXT OUTPUT
         ADD  A,"0"-" "
         CALL #BB5A               ;TXT OUTPUT
         LD   A,"0"
         LD   (DECHAR+1),A
         RET
;
;CONVD16B
;
CONVD16B
         PUSH DE
         PUSH BC
         LD   BC,D16BUFF
         LD   A,#37          ;SCF
         LD   (FIRSTB0),A
         CALL CONVDB0
         LD   A,#FF
         LD   (BC),A
         LD   HL,D16BUFF
         POP  BC
         POP  DE
         RET
CONVDB0  LD   DE,10000
         CALL CONVDB1
         LD   DE,1000
         CALL CONVDB1
         LD   DE,100
         CALL CONVDB1
         LD   DE,10
         CALL CONVDB1
         LD   DE,1
         LD   A,#B7          ;OR A
         LD   (FIRSTB0),A
CONVDB1  XOR  A
CONVDB2  SBC  HL,DE
         INC  A
         JR   NC,CONVDB2
         ADD  HL,DE
         ADD  A,"0"-1
         CP   "0"
         JR   NZ,CONVDB3
FIRSTB0  SCF
         JR   NC,CONVDB3
         LD   A," "
         LD   (BC),A
         INC  BC
         RET
CONVDB3  LD   (BC),A
         INC  BC
         LD   A,#B7          ;OR  A
         LD   (FIRSTB0),A
         RET
;
;CONVD16
;
D16BUFF  EQU  TAMPON         ;6
CONVD16
         PUSH DE
         PUSH BC
         LD   BC,D16BUFF
         LD   A,#37          ;SCF
         LD   (FIRST0),A
         CALL CONVD0
         LD   A,#FF
         LD   (BC),A
         LD   HL,D16BUFF
         POP  BC
         POP  DE
         RET
CONVD0   LD   DE,10000
         CALL CONVD1
         LD   DE,1000
         CALL CONVD1
         LD   DE,100
         CALL CONVD1
         LD   DE,10
         CALL CONVD1
         LD   DE,1
         LD   A,#B7          ;OR A
         LD   (FIRST0),A
CONVD1   XOR  A
CONVD2   SBC  HL,DE
         INC  A
         JR   NC,CONVD2
         ADD  HL,DE
         ADD  A,"0"-1
         CP   "0"
         JR   NZ,CONVD3
FIRST0   SCF
         RET  C
CONVD3   LD   (BC),A
         INC  BC
         LD   A,#B7          ;OR  A
         LD   (FIRST0),A
         RET
;
;MENUSEL
;Selection d'une option selon les parametres CHPNB et CHAMPS cree par MENUTEXT
;OKNULL  permet le retour sans selection
;
CHPSEL   DEFS 1         ;Champ selectionne courant
OKNULL   DEFB 0
;
MENUSEL
         PUSH HL
         PUSH BC
         PUSH IX
         XOR  A
         LD   (CHPSEL),A
         SCF
MENUSEL0
         CALL MFLECHE
         PUSH AF
         CALL GFLECH1
         SRL  H
         SRL  H
         INC  H
         LD   A,199
         SUB  L
         LD   L,A
         SRL  L
         SRL  L
         SRL  L
         INC  L
         LD   A,(NBCHP)
         SRL  A
         LD   B,A
         LD   IX,CHAMPS
MENUSEL1 LD   A,H
         SUB  (IX+01)
         JR   C,MENUSEL2
         LD   A,L
         SUB  (IX+00)
         JR   C,MENUSEL2
         LD   A,(IX+03)
         SUB  H
         JR   C,MENUSEL2
         LD   A,(IX+02)
         SUB  L
         JR   NC,MENUSEL3
MENUSEL2 INC  IX
         INC  IX
         INC  IX
         INC  IX
         DJNZ MENUSEL1
         LD   HL,CHPSEL
         LD   A,(HL)
         OR   A
         CALL NZ,INVCHP
         XOR  A
         LD   (HL),A
MENUSEL4 POP  AF
         JR   NC,MENUSEL0
         LD   A,(CHPSEL)
         LD   C,A
         LD   A,(OKNULL)
         OR   C
         JP   Z,MENUSEL0
         LD   A,C
         OR   A
         CALL NZ,INVCHP
         POP  IX
         POP  BC
         POP  HL
         RET
MENUSEL3 LD   A,(NBCHP)
         SRL  A
         SUB  B
         INC  A
         LD   HL,CHPSEL
         CP   (HL)
         JR   Z,MENUSEL4
         LD   B,(HL)
         LD   (HL),A
         LD   A,B
         OR   A
         CALL NZ,INVCHP
         LD   A,(HL)
         CALL INVCHP
         JR   MENUSEL4
;
;INVCHP
;Inverse le champ No A de la table CHAMPS
;
INVCHP
         PUSH BC
         PUSH DE
         PUSH HL
         PUSH AF
         LD   HL,CHAMPS
         DEC  A
         SLA  A
         CALL RDTAB16
         DEC  H
         DEC  L
         PUSH HL
         PUSH AF
         CALL #BC1A          ;SCR CHAR POSITION
         POP  AF
         PUSH HL
         POP  BC
         INC  A
         LD   HL,CHAMPS
         CALL RDTAB16
         POP  DE
         PUSH BC
         OR   A
         SBC  HL,DE
         LD   B,H
         SLA  B
         LD   C,L
         SLA  C
         SLA  C
         SLA  C
         PUSH BC
         CALL #BD19          ;MC WAIT FLYBACK
         CALL GFLECH2
         CALL MEMSCR
         POP  BC
         POP  HL
INVCHP0  PUSH BC
         PUSH HL
INVCHP1  LD   A,(HL)
         XOR  #C0
         LD   (HL),A
         INC  HL
         DJNZ INVCHP1
         POP  HL
         CALL NEXTLINE
         POP  BC
         DEC  C
         JR   NZ,INVCHP0
         CALL GFLECH2
         PUSH HL
         CALL SCRMEM
         POP  HL
         CALL FLON
         POP  AF
         POP  HL
         POP  DE
         POP  BC
         RET
;
;MENUSAVE
;
MENUSAVE
         PUSH DE
         PUSH HL
         PUSH BC
         PUSH AF
         LD   A,(TXTCOL)
         LD   H,A
         LD   A,(TXTROW)
         LD   L,A
         DEC  H
         DEC  L
         CALL #BC1A
         LD   A,(TXTNBROW)
         INC  A
         INC  A
         SLA  A
         SLA  A
         SLA  A
         LD   B,A
         LD   A,(TXTNBCOL)
         INC  A
         INC  A
         SLA  A
         LD   C,A
         LD   DE,(SAVEBUF)
MENUSAV0 PUSH BC
         PUSH HL
         LD   B,0
         LDIR
         POP  HL
         CALL NEXTLINE
         POP  BC
         DJNZ MENUSAV0
         POP  AF
         POP  BC
         POP  HL
         POP  DE
         RET
;
;MENUREST
;
MENUREST
         PUSH DE
         PUSH HL
         PUSH BC
         PUSH AF
         LD   A,(TXTCOL)
         LD   H,A
         LD   A,(TXTROW)
         LD   L,A
         DEC  H
         DEC  L
         CALL #BC1A
         LD   A,(TXTNBROW)
         INC  A
         INC  A
         SLA  A
         SLA  A
         SLA  A
         LD   B,A
         LD   A,(TXTNBCOL)
         INC  A
         INC  A
         SLA  A
         LD   C,A
         LD   DE,(SAVEBUF)
MENURES0 PUSH BC
         PUSH HL
         LD   B,0
         EX   DE,HL
         LDIR
         EX   DE,HL
         POP  HL
         CALL NEXTLINE
         POP  BC
         DJNZ MENURES0
         POP  AF
         POP  BC
         POP  HL
         POP  DE
         RET
;
;
;MENUCADR
;   Affiche le cadre du menu pointe par HL
;
TXTCADRE
         PUSH IX
         PUSH DE
         PUSH BC
         PUSH AF
         PUSH HL
         LD   A,(TXTCOL)
         LD   H,A
         LD   A,(TXTROW)
         LD   L,A
         DEC  H
         DEC  L
         CALL #BC1A          ;SCR CHAR POSITION
         LD   A,(TXTNBROW)
         LD   C,A
         LD   A,(TXTNBCOL)
         LD   B,A
         LD   IX,CHARHG
         CALL AFFCHAR
         PUSH HL
         PUSH BC
MENUC0   LD   IX,CHARH-16
         INC  HL
         INC  HL
         CALL RANDOM
         CALL AFFCHAR
         DJNZ MENUC0
         INC  HL
         INC  HL
         LD   IX,CHARHD
         CALL AFFCHAR
         POP  BC
         POP  HL
MENUC1   LD   DE,80
         ADD  HL,DE
         PUSH BC
         PUSH HL
         LD   IX,CHARG-16
         CALL RANDOM
         CALL AFFCHAR
MENUC10  INC  HL
         INC  HL
         LD   IX,CHARM
         CALL AFFCHAR
         DJNZ MENUC10
         INC  HL
         INC  HL
         LD   IX,CHARD-16
         CALL RANDOM
         CALL AFFCHAR
         POP  HL
         POP  BC
         DEC  C
         JR   NZ,MENUC1
         LD   DE,80
         ADD  HL,DE
         LD   IX,CHARBG
         CALL AFFCHAR
MENUC2   LD   IX,CHARB-16
         INC  HL
         INC  HL
         CALL RANDOM
         CALL AFFCHAR
         DJNZ MENUC2
         INC  HL
         INC  HL
         LD   IX,CHARBD
         CALL AFFCHAR
         POP  HL
         POP  AF
         POP  BC
         POP  DE
         POP  IX
         RET
;
RANDOM
         LD   DE,16
         LD   A,4
         CALL RND
RANDOM2  ADD  IX,DE
         DEC  A
         JR   NZ,RANDOM2
         RET
;
AFFCHAR
         PUSH HL
         PUSH DE
         PUSH BC
         PUSH IX
         LD   B,8
AFFCHAR0 LD   A,(IX+00)
         CALL AFFTRAN
         INC  HL
         INC  IX
         LD   A,(IX+00)
         CALL AFFTRAN
         LD   DE,#7FF
         ADD  HL,DE
         INC  IX
         DJNZ AFFCHAR0
         POP  IX
         POP  BC
         POP  DE
         POP  HL
         RET
;
AFFTRAN
         PUSH AF
         RRCA
         RRCA
         AND  #CC
         CPL
         AND  (HL)
         LD   (HL),A
         POP  AF
         OR   (HL)
         LD   (HL),A
         RET
;
;
;Definition des caracteres de bordure de parchemins
;
CHARHG   DEFB #00,#00,#00,#00,#00,#33,#00,#73
         DEFB #11,#73,#11,#F3,#11,#F3,#11,#F3
CHARHD   DEFB #00,#00,#00,#00,#33,#00,#B3,#00
         DEFB #B3,#22,#F3,#22,#F3,#22,#F3,#22
CHARBG   DEFB #11,#F3,#11,#F3,#11,#F3,#11,#73
         DEFB #00,#73,#00,#33,#00,#00,#00,#00
CHARBD   DEFB #F3,#22,#F3,#22,#F3,#22,#B3,#22
         DEFB #B3,#00,#33,#00,#00,#00,#00,#00
CHARH    DEFB #00,#00,#00,#00,#00,#00,#33,#33
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #00,#00,#00,#00,#33,#33,#F3,#F3
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #00,#00,#00,#00,#11,#33,#33,#F3
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #00,#00,#00,#00,#33,#00,#B3,#33
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
CHARG    DEFB #11,#F3,#11,#F3,#11,#F3,#11,#F3
         DEFB #33,#F3,#73,#F3,#73,#F3,#73,#F3
         DEFB #73,#F3,#73,#F3,#73,#F3,#33,#F3
         DEFB #11,#F3,#11,#F3,#11,#F3,#11,#F3
         DEFB #73,#F3,#73,#F3,#33,#F3,#11,#73
         DEFB #33,#F3,#73,#F3,#73,#F3,#73,#F3
         DEFB #11,#F3,#11,#F3,#11,#F3,#33,#F3
         DEFB #73,#F3,#33,#F3,#11,#F3,#11,#F3
CHARD    DEFB #F3,#22,#F3,#22,#F3,#22,#F3,#22
         DEFB #F3,#22,#F3,#22,#F3,#22,#F3,#22
         DEFB #F3,#B3,#F3,#B3,#F3,#B3,#F3,#B3
         DEFB #F3,#B3,#F3,#B3,#F3,#B3,#F3,#B3
         DEFB #F3,#22,#F3,#22,#F3,#33,#F3,#B3
         DEFB #F3,#33,#F3,#22,#F3,#22,#F3,#22
         DEFB #F3,#B3,#F3,#B3,#F3,#B3,#F3,#33
         DEFB #B3,#22,#F3,#33,#F3,#B3,#F3,#B3
CHARB    DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #F3,#F3,#33,#33,#00,#00,#00,#00
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #33,#33,#00,#00,#00,#00,#00,#00
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #F3,#33,#33,#22,#00,#00,#00,#00
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #33,#73,#00,#33,#00,#00,#00,#00
CHARM    DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3
         DEFB #F3,#F3,#F3,#F3,#F3,#F3,#F3,#F3





